# frozen_string_literal: true

module RTX
  module API
    class Collection
      attr_accessor :client, :resource_name, :response, :options

      def initialize(client, resource_name, attrs = {})
        @client = client
        @resource_name = resource_name.to_sym
        @options = symbolize_hash(attrs)
        @response = {}
      end

      # Creates a new resource via POST and returns it on success
      def create!(attrs = {})
        client.authenticate if !client.authenticated?
        post(symbolize_hash(attrs))
      end

      # Gets an individual resource returns an object instead of an array
      def detail!(attrs = {})
        client.authenticate if !client.authenticated?
        detail(symbolize_hash(attrs))
      end

      # Updates a resource via PUT and returns it on success
      def update!(attrs = {})
        client.authenticate if !client.authenticated?
        put(symbolize_hash(attrs))
      end

      # Updates a resource via PATCH and returns it on success
      def patch!(attrs = {})
        client.authenticate if !client.authenticated?
        patch(symbolize_hash(attrs))
      end

      # Deletes a resource via DELETE and returns true on success
      def delete!(attrs = {})
        client.authenticate if !client.authenticated?
        delete(symbolize_hash(attrs))
      end

      def get!(attrs = {}, file_type = 'json')
        client.authenticate if !client.authenticated?
        get(symbolize_hash(attrs), file_type)
      end

      # Chainable method that allows you to set the per page number of
      # the collection for your request
      def per_page(num)
        clear if !num.nil?
        @options[:per_page] = num
        self
      end

      # Chainable method that allows you to set the page number of the collection for your request
      def page(num)
        clear if !num.nil?
        @options[:page] = num
        self
      end

      # Returns all data associated with the existing response
      def data
        client.authenticate if !client.authenticated?
        collection if !has_response?
        response[:_embedded]&.[](resource_name) || response
      end

      # Returns the metadata about the current response
      def meta
        client.authenticate if !client.authenticated?
        collection if !has_response?
        response.reject { |key, _| key == :_embedded }
      end

      # For moving forward one page with the collection
      def next
        page(options[:page] += 1) if has_next?
        self
      end

      # For moving backward one page with the collection
      def prev
        page(options[:page] -= 1) if has_previous?
        self
      end

      # For moving to the last page of the collection
      def last
        page(meta[:_total_pages])
        self
      end

      # For moving to the first page of the collection
      def first
        page(1)
        self
      end

      # Responds true if the collection has another page ahead of it
      def has_next?
        current_page < meta[:_total_pages]
      end

      # Responds true if the collection has a previous one
      def has_previous?
        current_page > 1
      end

      # Allows you to loop through all of the pages and retrieve the records
      def all_pages(initial_page = 1, &block)
        page(initial_page)

        pages = (current_page..meta[:_total_pages]).to_a
        pages.each do |page_num|
          block.call(page(page_num).data)
        end
      end

      # Allows you to loop through all of the resources
      # within the pages specified and retrieve the records
      def all_resources(initial_page = 1, &block)
        all_pages(initial_page) do |page|
          page.each do |resource|
            block.call(resource)
          end
        end
      end

      protected

      def clear
        @response = {}
      end

      def current_page
        @options[:page] = options[:page].nil? ? meta[:_page] : options[:page]
        options[:page]
      end

      def collection
        @response = client.collection(resource_name, options)
      end

      def detail(attrs = {})
        resource_id = attrs[:id]
        client.detail(resource_name, resource_id, attrs)
      end

      def post(attrs = {})
        client.post(resource_name, attrs)
      end

      def put(attrs = {})
        resource_id = attrs[:id]
        client.put(resource_name, resource_id, attrs)
      end

      def patch(attrs = {})
        resource_id = attrs[:id]
        client.patch(resource_name, resource_id, attrs)
      end

      def delete(attrs = {})
        resource_id = attrs[:id]
        client.delete(resource_name, resource_id)
      end

      def get(attrs = {}, file_type = 'json')
        client.get(resource_name, attrs, file_type)
      end

      def has_response?
        response != {}
      end

      def symbolize_hash(hash)
        Hash[hash.map { |(key, value)| [key.to_sym, value] }]
      end
    end
  end
end
