# frozen_string_literal: true

module RTX
  module API
    class CollectionV2
      attr_accessor :client, :resource_name, :response, :options

      def initialize(client, resource_name, attrs = {})
        @client = client
        @resource_name = resource_name.to_sym
        @options = symbolize_hash(attrs)
        @response = {}
      end

      # Chainable method that allows you to set
      # the per page number of the collection for your request
      def per_page(num)
        clear if !num.nil?
        @options[:per_page] = num
        self
      end

      # Returns all data associated with the existing response
      def data
        client.authenticate if !client.authenticated?
        collection if !has_response?
        response[:data]
      end

      # Returns the paging information about the current response
      def paging
        client.authenticate if !client.authenticated?
        collection if !has_response?
        response[:paging]
      end

      # For moving forward one page with the collection
      def next
        next_page(after_token) if has_next?
        self
      end

      # For moving backward one page with the collection
      def prev
        previous_page(before_token) if has_previous?
        self
      end

      # Responds true if the collection has another page ahead of it
      def has_next?
        !after_token.nil?
      end

      # Responds true if the collection has a previous one
      def has_previous?
        !before_token.nil?
      end

      # Allows you to loop through all of the pages and retrieve the records
      def all_pages(&block)
        loop do
          # Return first page
          block.call(data)

          # No need to continue if all data is retrieved
          break unless has_next?

          # Navigate to the next page
          self.next
        end
      end

      # Allows you to loop through all of
      # the resources within the pages specified and retrieve the records
      def all_resources(&block)
        all_pages do |page|
          page.each do |resource|
            block.call(resource)
          end
        end
      end

      protected

      # Chainable method that allows you to get the next page for a given after token
      def next_page(after_token)
        clear
        @options[:after] = after_token
        self
      end

      # Chainable method that allows you to get the previous page for a given before token
      def previous_page(before_token)
        clear
        @options[:before] = before_token
        self
      end

      def after_token
        return nil if paging.nil?
        return nil if paging[:cursors].nil?

        paging[:cursors][:after]
      end

      def before_token
        return nil if paging.nil?
        return nil if paging[:cursors].nil?

        paging[:cursors][:before]
      end

      def clear
        @response = {}
      end

      def collection
        @response = client.collection(resource_name, options)
      end

      def has_response?
        response != {}
      end

      def symbolize_hash(hash)
        Hash[hash.map { |(key, value)| [key.to_sym, value] }]
      end
    end
  end
end
